@@ -5,8 +5,7 @@ This file is part of PyXMMS.
 
 PyXMMS is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+the Free Software Foundation; version 2 dated June, 1991.
 
 PyXMMS is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -444,7 +443,7 @@ The code fragment that answers to this request in XMMS 1.2.6 is:\n\
           ...\n\
 \n\
 I suspect it is the version of the protocol in use through the\n\
-XMMS control socket (which is the way libxmms talks to XMMS, by the\n\
+XMMS control socket (which is how libxmms talks to XMMS, by the\n\
 way).\n\
 \n\
 If you understand this better than I, please let me know.");
