@@ -1,5 +1,5 @@
 # control.py --- Python interface to libxmms -- control module.
-# Copyright (c) 2002, 2003 Florent Rougon
+# Copyright (c) 2002, 2003, 2005 Florent Rougon
 #
 # This file is part of PyXMMS.
 #
@@ -199,9 +199,10 @@ Exceptions specific to this module
 ----------------------------------
 
 ExecutableNotFound
+RequestedSessionDoesNotComeUp
 InvalidFadeOutAction
 
-Both exceptions are subclasses of xmms.error.
+They are all subclasses of xmms.error.
 
 """
 
@@ -214,6 +215,10 @@ class ExecutableNotFound(common.error):
     """Exception raised when the XMMS executable can't be found."""
     ExceptionShortDescription = "Executable not found"
 
+class RequestedSessionDoesNotComeUp(common.error):
+    """Exception raised when a started XMMS session still doesn't answer after a specified timeout."""
+    ExceptionShortDescription = "Requested session doesn't come up"
+
 class InvalidFadeOutAction(common.error):
     """Exception raised when fade_out is given an invalid action."""
     ExceptionShortDescription = "Invalid action"
@@ -292,22 +297,43 @@ def enqueue_and_play(seq, session=0):
 
 
 def enqueue_and_play_launch_if_session_not_started(seq, xmms_prg="xmms",
-                                                   session=0):
+                                                   session=0,
+                                                   poll_delay=0.1,
+                                                   timeout=10.0):
     """Add files/URLs to the playlist and start playing from the first one.
 
-    seq      -- a sequence of files/URLs
-    xmms_prg -- the name (absolute or looked up in the PATH)
-                of an XMMS binary to invoke in case the specified
-                session is not running
-    session  -- the XMMS session to act on
+    seq        -- a sequence of files/URLs
+    xmms_prg   -- the name (absolute or looked up in the PATH)
+                  of an XMMS binary to invoke in case the specified
+                  session is not running
+    session    -- the XMMS session to act on
+    poll_delay -- poll delay, in seconds (float, see below)
+    timeout    -- timeout while polling, in seconds (float, see below)
 
     This function is identical to enqueue_and_play except that it
     spawns an XMMS process if the requested session is not running.
 
+    When it does spawn an XMMS process, it has to wait until XMMS is
+    ready to handle requests (here, the first request will be
+    enqueue_and_play). It will therefore check every 'poll_delay'
+    seconds whether the requested session is ready, and abort after
+    'timeout' seconds of unsuccessful checks, raising
+    xmms.control.RequestedSessionDoesNotComeUp. If we get that far,
+    it may be that your system is very slow, or more likely that the
+    XMMS session that was started by this function is not the one
+    numbered 'session': there is currently no way to start an XMMS
+    session for a chosen number; we have to guess the number of the
+    session that will be started...
+
     Return None.
 
-    Notable exception: xmms.control.ExecutableNotFound is raised if
-    'xmms_prg' can't be found, read and executed.
+    Notable exceptions:
+        - xmms.control.ExecutableNotFound is raised if 'xmms_prg'
+          can't be found, read and executed.
+        - xmms.control.RequestedSessionDoesNotComeUp is raised if the
+          requested session is still unable to handle requests
+          'timeout' seconds after the XMMS process was started by
+          this function.
 
     """
     if not is_running(session):
@@ -322,15 +348,24 @@ def enqueue_and_play_launch_if_session_not_started(seq, xmms_prg="xmms",
             # We are in the child, me MUST NOT trigger any exception (look at
             # _spawnvef in Python's os.py).
             try:
-                os.execvp(abs_prog_name, (abs_prog_name, "-p") + tuple(seq))
+                os.execvp(abs_prog_name, (abs_prog_name,))
             except:
                 # We cannot know in the father process if the child's execvp
                 # failed, but AFAIK, there is no simple solution since I don't
                 # want the father to wait for the child's completion. init
                 # will be a good father. :-)
                 os._exit(127)
-    else:
-        enqueue_and_play(seq, session) 
+
+        else:                           # We are in the father
+            start_time = time.time()
+            while not is_running(session):
+                if time.time() - start_time >= timeout:
+                    raise RequestedSessionDoesNotComeUp(
+                        "session %u still unavailable after %.2f seconds "
+                        "timeout" % (session, timeout))
+                time.sleep(poll_delay)
+
+    enqueue_and_play(seq, session)
 
 
 def fade_out(action="stop", nb_steps=20, step_duration=0.5, restore_volume=1,
